from zetasql.wasi._pb2.zetasql.compliance import known_error_pb2 as _known_error_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ComplianceTestsLabels(_message.Message):
    __slots__ = ("test_cases",)
    TEST_CASES_FIELD_NUMBER: _ClassVar[int]
    test_cases: _containers.RepeatedCompositeFieldContainer[ComplianceTestCaseLabels]
    def __init__(self, test_cases: _Optional[_Iterable[_Union[ComplianceTestCaseLabels, _Mapping]]] = ...) -> None: ...

class ComplianceTestCaseLabels(_message.Message):
    __slots__ = ("test_name", "test_query", "param", "test_error_mode", "compliance_labels", "test_shard", "test_location")
    class Param(_message.Message):
        __slots__ = ("param_name", "param_value_literal")
        PARAM_NAME_FIELD_NUMBER: _ClassVar[int]
        PARAM_VALUE_LITERAL_FIELD_NUMBER: _ClassVar[int]
        param_name: str
        param_value_literal: str
        def __init__(self, param_name: _Optional[str] = ..., param_value_literal: _Optional[str] = ...) -> None: ...
    class Location(_message.Message):
        __slots__ = ("file", "line")
        FILE_FIELD_NUMBER: _ClassVar[int]
        LINE_FIELD_NUMBER: _ClassVar[int]
        file: str
        line: int
        def __init__(self, file: _Optional[str] = ..., line: _Optional[int] = ...) -> None: ...
    TEST_NAME_FIELD_NUMBER: _ClassVar[int]
    TEST_QUERY_FIELD_NUMBER: _ClassVar[int]
    PARAM_FIELD_NUMBER: _ClassVar[int]
    TEST_ERROR_MODE_FIELD_NUMBER: _ClassVar[int]
    COMPLIANCE_LABELS_FIELD_NUMBER: _ClassVar[int]
    TEST_SHARD_FIELD_NUMBER: _ClassVar[int]
    TEST_LOCATION_FIELD_NUMBER: _ClassVar[int]
    test_name: str
    test_query: str
    param: _containers.RepeatedCompositeFieldContainer[ComplianceTestCaseLabels.Param]
    test_error_mode: _known_error_pb2.KnownErrorMode
    compliance_labels: _containers.RepeatedScalarFieldContainer[str]
    test_shard: int
    test_location: ComplianceTestCaseLabels.Location
    def __init__(self, test_name: _Optional[str] = ..., test_query: _Optional[str] = ..., param: _Optional[_Iterable[_Union[ComplianceTestCaseLabels.Param, _Mapping]]] = ..., test_error_mode: _Optional[_Union[_known_error_pb2.KnownErrorMode, str]] = ..., compliance_labels: _Optional[_Iterable[str]] = ..., test_shard: _Optional[int] = ..., test_location: _Optional[_Union[ComplianceTestCaseLabels.Location, _Mapping]] = ...) -> None: ...



"""PreparedQuery wrapper for automatic resource management.

Provides Java-style PreparedQuery with context manager support
for automatic cleanup of server-side resources.
"""

import contextlib
from typing import Any

import zetasql.types
from zetasql.core import IllegalStateError, InvalidArgumentError, ZetaSqlLocalService


class PreparedQuery:
    """Context manager for prepared queries with automatic cleanup.

    Equivalent to Java's PreparedQuery with AutoCloseable support.
    Automatically releases server-side resources when exiting context.

    Example:
        >>> with PreparedQuery.builder() \\
        ...         .set_sql("SELECT * FROM Orders") \\
        ...         .set_analyzer_options(options) \\
        ...         .set_catalog(catalog) \\
        ...         .prepare() as query:
        ...     result = query.execute()
        ...     # Automatically cleaned up on exit
    """

    def __init__(self, service: ZetaSqlLocalService, prepared_id: int, columns: list):
        """Initialize PreparedQuery.

        Args:
            service: ZetaSqlLocalService instance
            prepared_id: Server-side prepared query ID
            columns: List of output column metadata
        """
        self._service = service
        self._prepared_id = prepared_id
        self._columns = columns
        self._closed = False

    @property
    def prepared_query_id(self) -> int:
        """Get prepared query ID.

        Returns:
            Server-side prepared query identifier
        """
        return self._prepared_id

    @property
    def columns(self) -> list:
        """Get output column metadata.

        Returns:
            List of column descriptors (name, type, etc.)
        """
        return self._columns

    def execute(self, parameters: dict[str, Any] | None = None, table_content: dict[str, Any] | None = None):
        """Execute the prepared query.

        Args:
            parameters: Optional query parameters (for parameterized queries)
            table_content: Optional table content override

        Returns:
            EvaluateQueryResponse with query results

        Raises:
            IllegalStateError: If query has already been closed

        Example:
            >>> response = query.execute()
            >>> for row in response.content.table_data.row:
            ...     print(row)
        """
        if self._closed:
            raise IllegalStateError("PreparedQuery already closed. Cannot execute closed query.")

        response = self._service.evaluate_query(
            prepared_query_id=self._prepared_id,
            params=parameters or {},
            table_content=table_content or {},
        )
        return response

    def close(self):
        """Release server-side resources. Safe to call multiple times."""
        if not self._closed:
            try:
                self._service.unprepare_query(prepared_query_id=self._prepared_id)
            finally:
                self._closed = True

    def __enter__(self):
        """Enter context manager."""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Exit context manager and cleanup resources."""
        self.close()
        return False

    def __del__(self):
        """Cleanup on garbage collection. Explicit close() or context manager usage is preferred."""
        if not self._closed:
            with contextlib.suppress(Exception):
                self.close()

    @staticmethod
    def builder():
        """Create a new PreparedQueryBuilder.

        Returns:
            PreparedQueryBuilder instance for fluent API

        Example:
            >>> query = PreparedQuery.builder() \\
            ...     .set_sql("SELECT * FROM Orders") \\
            ...     .set_analyzer_options(options) \\
            ...     .prepare()
        """
        return PreparedQueryBuilder()


class PreparedQueryBuilder:
    """Builder for PreparedQuery with fluent API.

    Equivalent to Java's PreparedQuery.Builder pattern.
    Provides method chaining for convenient query preparation.

    Example:
        >>> builder = PreparedQueryBuilder()
        >>> builder.set_sql("SELECT id FROM Orders") \\
        ...        .set_analyzer_options(options) \\
        ...        .set_catalog(catalog) \\
        ...        .set_table_content({"Orders": orders_data})
        >>> with builder.prepare() as query:
        ...     result = query.execute()
    """

    def __init__(self):
        """Initialize empty builder."""
        self._sql = None
        self._options = None
        self._catalog = None
        self._registered_catalog_id = None
        self._table_content = None
        self._service = None

    def set_sql(self, sql: str):
        """Set SQL query string.

        Args:
            sql: SQL query to prepare

        Returns:
            Self for method chaining
        """
        self._sql = sql
        return self

    def set_analyzer_options(self, options: zetasql.types.AnalyzerOptions):
        """Set analyzer options.

        Args:
            options: Analyzer options for query analysis

        Returns:
            Self for method chaining
        """
        self._options = options
        return self

    def set_catalog(self, catalog: zetasql.types.SimpleCatalog):
        """Set unregistered catalog.

        Args:
            catalog: Simple catalog with tables and functions

        Returns:
            Self for method chaining

        Note:
            Mutually exclusive with set_registered_catalog_id()
        """
        self._catalog = catalog
        return self

    def set_registered_catalog_id(self, catalog_id: int):
        """Set registered catalog ID.

        Args:
            catalog_id: ID of previously registered catalog

        Returns:
            Self for method chaining

        Note:
            Mutually exclusive with set_catalog()
        """
        self._registered_catalog_id = catalog_id
        return self

    def set_table_content(self, table_content: dict[str, Any]):
        """Set table content for execution.

        Args:
            table_content: Dict mapping table names to TableContent objects

        Returns:
            Self for method chaining

        Note:
            Only valid with set_catalog(), not with registered catalog
        """
        self._table_content = table_content
        return self

    def set_service(self, service):
        """Set custom service instance (optional).

        Args:
            service: ZetaSqlLocalService instance

        Returns:
            Self for method chaining

        Note:
            If not set, uses singleton instance
        """
        self._service = service
        return self

    def prepare(self) -> PreparedQuery:
        """Prepare the query and return PreparedQuery instance.

        Returns:
            PreparedQuery instance ready for execution

        Raises:
            InvalidArgumentError: If required parameters are missing or invalid

        Example:
            >>> query = builder.prepare()
            >>> try:
            ...     result = query.execute()
            ... finally:
            ...     query.close()
        """
        # Validation - SQL is required
        if not self._sql:
            raise InvalidArgumentError("SQL must be set")

        # Validate SQL is not just whitespace
        if not self._sql.strip():
            raise InvalidArgumentError("SQL string must not be empty or whitespace-only")

        # Mutually exclusive parameters
        if self._catalog and self._registered_catalog_id:
            raise InvalidArgumentError("Cannot provide both catalog and registered_catalog_id. Use one or the other.")

        # table_content requires simple_catalog
        if self._table_content and self._registered_catalog_id:
            raise InvalidArgumentError(
                "Cannot use table_content with registered catalog. table_content requires simple_catalog.",
            )

        service = self._service or ZetaSqlLocalService.get_instance()

        response = service.prepare_query(
            sql=self._sql,
            options=self._options,
            simple_catalog=self._catalog,
            registered_catalog_id=self._registered_catalog_id,
            table_content=self._table_content or {},
        )

        return PreparedQuery(
            service=service,
            prepared_id=response.prepared.prepared_query_id,
            columns=response.prepared.columns,
        )



from google.protobuf import descriptor_pb2 as _descriptor_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor
FORMAT_FIELD_NUMBER: _ClassVar[int]
format: _descriptor.FieldDescriptor
TYPE_FIELD_NUMBER: _ClassVar[int]
type: _descriptor.FieldDescriptor
ENCODING_FIELD_NUMBER: _ClassVar[int]
encoding: _descriptor.FieldDescriptor
USE_DEFAULTS_FIELD_NUMBER: _ClassVar[int]
use_defaults: _descriptor.FieldDescriptor
USE_FIELD_DEFAULTS_FIELD_NUMBER: _ClassVar[int]
use_field_defaults: _descriptor.FieldDescriptor

class FieldFormat(_message.Message):
    __slots__ = ()
    class Format(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        DEFAULT_FORMAT: _ClassVar[FieldFormat.Format]
        DATE: _ClassVar[FieldFormat.Format]
        TIMESTAMP_SECONDS: _ClassVar[FieldFormat.Format]
        TIMESTAMP_MILLIS: _ClassVar[FieldFormat.Format]
        TIMESTAMP_MICROS: _ClassVar[FieldFormat.Format]
        TIMESTAMP_NANOS: _ClassVar[FieldFormat.Format]
        TIMESTAMP_PICOS: _ClassVar[FieldFormat.Format]
        TIMESTAMP: _ClassVar[FieldFormat.Format]
        DATE_DECIMAL: _ClassVar[FieldFormat.Format]
        TIME_MICROS: _ClassVar[FieldFormat.Format]
        DATETIME_MICROS: _ClassVar[FieldFormat.Format]
        ST_GEOGRAPHY_ENCODED: _ClassVar[FieldFormat.Format]
        NUMERIC: _ClassVar[FieldFormat.Format]
        BIGNUMERIC: _ClassVar[FieldFormat.Format]
        JSON: _ClassVar[FieldFormat.Format]
        INTERVAL: _ClassVar[FieldFormat.Format]
        TOKENLIST: _ClassVar[FieldFormat.Format]
        RANGE_DATES_ENCODED: _ClassVar[FieldFormat.Format]
        RANGE_DATETIMES_ENCODED: _ClassVar[FieldFormat.Format]
        RANGE_TIMESTAMPS_ENCODED: _ClassVar[FieldFormat.Format]
        UUID: _ClassVar[FieldFormat.Format]
        __FieldFormat_Type__switch_must_have_a_default__: _ClassVar[FieldFormat.Format]
    DEFAULT_FORMAT: FieldFormat.Format
    DATE: FieldFormat.Format
    TIMESTAMP_SECONDS: FieldFormat.Format
    TIMESTAMP_MILLIS: FieldFormat.Format
    TIMESTAMP_MICROS: FieldFormat.Format
    TIMESTAMP_NANOS: FieldFormat.Format
    TIMESTAMP_PICOS: FieldFormat.Format
    TIMESTAMP: FieldFormat.Format
    DATE_DECIMAL: FieldFormat.Format
    TIME_MICROS: FieldFormat.Format
    DATETIME_MICROS: FieldFormat.Format
    ST_GEOGRAPHY_ENCODED: FieldFormat.Format
    NUMERIC: FieldFormat.Format
    BIGNUMERIC: FieldFormat.Format
    JSON: FieldFormat.Format
    INTERVAL: FieldFormat.Format
    TOKENLIST: FieldFormat.Format
    RANGE_DATES_ENCODED: FieldFormat.Format
    RANGE_DATETIMES_ENCODED: FieldFormat.Format
    RANGE_TIMESTAMPS_ENCODED: FieldFormat.Format
    UUID: FieldFormat.Format
    __FieldFormat_Type__switch_must_have_a_default__: FieldFormat.Format
    def __init__(self) -> None: ...

class DeprecatedEncoding(_message.Message):
    __slots__ = ()
    class Encoding(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        DEFAULT_ENCODING: _ClassVar[DeprecatedEncoding.Encoding]
        DATE_DECIMAL: _ClassVar[DeprecatedEncoding.Encoding]
        DATE_PACKED32: _ClassVar[DeprecatedEncoding.Encoding]
        __FieldFormat_Encoding__switch_must_have_a_default__: _ClassVar[DeprecatedEncoding.Encoding]
    DEFAULT_ENCODING: DeprecatedEncoding.Encoding
    DATE_DECIMAL: DeprecatedEncoding.Encoding
    DATE_PACKED32: DeprecatedEncoding.Encoding
    __FieldFormat_Encoding__switch_must_have_a_default__: DeprecatedEncoding.Encoding
    def __init__(self) -> None: ...



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class ResolvedNodeKind(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    RESOLVED_LITERAL: _ClassVar[ResolvedNodeKind]
    RESOLVED_PARAMETER: _ClassVar[ResolvedNodeKind]
    RESOLVED_EXPRESSION_COLUMN: _ClassVar[ResolvedNodeKind]
    RESOLVED_CATALOG_COLUMN_REF: _ClassVar[ResolvedNodeKind]
    RESOLVED_COLUMN_REF: _ClassVar[ResolvedNodeKind]
    RESOLVED_GROUPING_SET_MULTI_COLUMN: _ClassVar[ResolvedNodeKind]
    RESOLVED_CONSTANT: _ClassVar[ResolvedNodeKind]
    RESOLVED_SYSTEM_VARIABLE: _ClassVar[ResolvedNodeKind]
    RESOLVED_INLINE_LAMBDA: _ClassVar[ResolvedNodeKind]
    RESOLVED_SEQUENCE: _ClassVar[ResolvedNodeKind]
    RESOLVED_FILTER_FIELD_ARG: _ClassVar[ResolvedNodeKind]
    RESOLVED_FILTER_FIELD: _ClassVar[ResolvedNodeKind]
    RESOLVED_FUNCTION_CALL: _ClassVar[ResolvedNodeKind]
    RESOLVED_AGGREGATE_FUNCTION_CALL: _ClassVar[ResolvedNodeKind]
    RESOLVED_ANALYTIC_FUNCTION_CALL: _ClassVar[ResolvedNodeKind]
    RESOLVED_EXTENDED_CAST_ELEMENT: _ClassVar[ResolvedNodeKind]
    RESOLVED_EXTENDED_CAST: _ClassVar[ResolvedNodeKind]
    RESOLVED_CAST: _ClassVar[ResolvedNodeKind]
    RESOLVED_MAKE_STRUCT: _ClassVar[ResolvedNodeKind]
    RESOLVED_MAKE_PROTO: _ClassVar[ResolvedNodeKind]
    RESOLVED_MAKE_PROTO_FIELD: _ClassVar[ResolvedNodeKind]
    RESOLVED_GET_STRUCT_FIELD: _ClassVar[ResolvedNodeKind]
    RESOLVED_GET_PROTO_FIELD: _ClassVar[ResolvedNodeKind]
    RESOLVED_GET_JSON_FIELD: _ClassVar[ResolvedNodeKind]
    RESOLVED_GET_ROW_FIELD: _ClassVar[ResolvedNodeKind]
    RESOLVED_FLATTEN: _ClassVar[ResolvedNodeKind]
    RESOLVED_FLATTENED_ARG: _ClassVar[ResolvedNodeKind]
    RESOLVED_REPLACE_FIELD_ITEM: _ClassVar[ResolvedNodeKind]
    RESOLVED_REPLACE_FIELD: _ClassVar[ResolvedNodeKind]
    RESOLVED_GET_PROTO_ONEOF: _ClassVar[ResolvedNodeKind]
    RESOLVED_SUBQUERY_EXPR: _ClassVar[ResolvedNodeKind]
    RESOLVED_WITH_EXPR: _ClassVar[ResolvedNodeKind]
    RESOLVED_EXECUTE_AS_ROLE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_MODEL: _ClassVar[ResolvedNodeKind]
    RESOLVED_CONNECTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_DESCRIPTOR: _ClassVar[ResolvedNodeKind]
    RESOLVED_SINGLE_ROW_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_UNSET_ARGUMENT_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_TABLE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_JOIN_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_ARRAY_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_COLUMN_HOLDER: _ClassVar[ResolvedNodeKind]
    RESOLVED_FILTER_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_GROUPING_CALL: _ClassVar[ResolvedNodeKind]
    RESOLVED_GROUPING_SET_LIST: _ClassVar[ResolvedNodeKind]
    RESOLVED_GROUPING_SET_PRODUCT: _ClassVar[ResolvedNodeKind]
    RESOLVED_GROUPING_SET: _ClassVar[ResolvedNodeKind]
    RESOLVED_ROLLUP: _ClassVar[ResolvedNodeKind]
    RESOLVED_CUBE: _ClassVar[ResolvedNodeKind]
    RESOLVED_AGGREGATE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_ANONYMIZED_AGGREGATE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_DIFFERENTIAL_PRIVACY_AGGREGATE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_AGGREGATION_THRESHOLD_AGGREGATE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_SET_OPERATION_ITEM: _ClassVar[ResolvedNodeKind]
    RESOLVED_SET_OPERATION_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_ORDER_BY_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_LIMIT_OFFSET_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_WITH_REF_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_ANALYTIC_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_SAMPLE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_COMPUTED_COLUMN: _ClassVar[ResolvedNodeKind]
    RESOLVED_DEFERRED_COMPUTED_COLUMN: _ClassVar[ResolvedNodeKind]
    RESOLVED_ORDER_BY_ITEM: _ClassVar[ResolvedNodeKind]
    RESOLVED_COLUMN_ANNOTATIONS: _ClassVar[ResolvedNodeKind]
    RESOLVED_GENERATED_COLUMN_INFO: _ClassVar[ResolvedNodeKind]
    RESOLVED_COLUMN_DEFAULT_VALUE: _ClassVar[ResolvedNodeKind]
    RESOLVED_COLUMN_DEFINITION: _ClassVar[ResolvedNodeKind]
    RESOLVED_PRIMARY_KEY: _ClassVar[ResolvedNodeKind]
    RESOLVED_FOREIGN_KEY: _ClassVar[ResolvedNodeKind]
    RESOLVED_CHECK_CONSTRAINT: _ClassVar[ResolvedNodeKind]
    RESOLVED_OUTPUT_COLUMN: _ClassVar[ResolvedNodeKind]
    RESOLVED_OUTPUT_SCHEMA: _ClassVar[ResolvedNodeKind]
    RESOLVED_PROJECT_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_TVFSCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_GROUP_ROWS_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_FUNCTION_ARGUMENT: _ClassVar[ResolvedNodeKind]
    RESOLVED_EXPLAIN_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_STATEMENT_WITH_PIPE_OPERATORS_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_QUERY_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_GENERALIZED_QUERY_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_MULTI_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_WITH_ENTRY_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_DATABASE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_INDEX_ITEM: _ClassVar[ResolvedNodeKind]
    RESOLVED_UNNEST_ITEM: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_INDEX_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_SCHEMA_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_EXTERNAL_SCHEMA_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_TABLE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_TABLE_AS_SELECT_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_MODEL_ALIASED_QUERY: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_MODEL_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_VIEW_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_WITH_PARTITION_COLUMNS: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_SNAPSHOT_TABLE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_EXTERNAL_TABLE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_EXPORT_MODEL_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_EXPORT_DATA_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_EXPORT_METADATA_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_DEFINE_TABLE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_DESCRIBE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_SHOW_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_BEGIN_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_SET_TRANSACTION_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_COMMIT_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ROLLBACK_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_START_BATCH_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_RUN_BATCH_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ABORT_BATCH_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_DROP_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_DROP_MATERIALIZED_VIEW_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_DROP_SNAPSHOT_TABLE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_RECURSIVE_REF_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_RECURSION_DEPTH_MODIFIER: _ClassVar[ResolvedNodeKind]
    RESOLVED_RECURSIVE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_WITH_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_WITH_ENTRY: _ClassVar[ResolvedNodeKind]
    RESOLVED_OPTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_WINDOW_PARTITIONING: _ClassVar[ResolvedNodeKind]
    RESOLVED_WINDOW_ORDERING: _ClassVar[ResolvedNodeKind]
    RESOLVED_WINDOW_FRAME: _ClassVar[ResolvedNodeKind]
    RESOLVED_ANALYTIC_FUNCTION_GROUP: _ClassVar[ResolvedNodeKind]
    RESOLVED_WINDOW_FRAME_EXPR: _ClassVar[ResolvedNodeKind]
    RESOLVED_DMLVALUE: _ClassVar[ResolvedNodeKind]
    RESOLVED_DMLDEFAULT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ASSERT_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ASSERT_ROWS_MODIFIED: _ClassVar[ResolvedNodeKind]
    RESOLVED_ON_CONFLICT_CLAUSE: _ClassVar[ResolvedNodeKind]
    RESOLVED_INSERT_ROW: _ClassVar[ResolvedNodeKind]
    RESOLVED_INSERT_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_DELETE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_UPDATE_ITEM: _ClassVar[ResolvedNodeKind]
    RESOLVED_UPDATE_ITEM_ELEMENT: _ClassVar[ResolvedNodeKind]
    RESOLVED_UPDATE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_MERGE_WHEN: _ClassVar[ResolvedNodeKind]
    RESOLVED_MERGE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_TRUNCATE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_OBJECT_UNIT: _ClassVar[ResolvedNodeKind]
    RESOLVED_PRIVILEGE: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRANT_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_REVOKE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_DATABASE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_INDEX_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_MATERIALIZED_VIEW_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_APPROX_VIEW_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_SCHEMA_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_EXTERNAL_SCHEMA_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_MODEL_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_TABLE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_VIEW_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_SET_OPTIONS_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_SUB_ENTITY_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_ADD_SUB_ENTITY_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_DROP_SUB_ENTITY_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_ADD_COLUMN_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_ADD_COLUMN_IDENTIFIER_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_REBUILD_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_ADD_CONSTRAINT_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_DROP_CONSTRAINT_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_DROP_PRIMARY_KEY_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_COLUMN_OPTIONS_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_COLUMN_DROP_NOT_NULL_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_COLUMN_DROP_GENERATED_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_COLUMN_SET_GENERATED_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_COLUMN_SET_DATA_TYPE_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_COLUMN_SET_DEFAULT_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_COLUMN_DROP_DEFAULT_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_DROP_COLUMN_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_RENAME_COLUMN_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_SET_AS_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_SET_COLLATE_CLAUSE: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_TABLE_SET_OPTIONS_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_RENAME_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_PRIVILEGE_RESTRICTION_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_ROW_ACCESS_POLICY_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_DROP_PRIVILEGE_RESTRICTION_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_DROP_ROW_ACCESS_POLICY_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_DROP_INDEX_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRANT_TO_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_RESTRICT_TO_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_ADD_TO_RESTRICTEE_LIST_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_REMOVE_FROM_RESTRICTEE_LIST_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_FILTER_USING_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_REVOKE_FROM_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_RENAME_TO_ACTION: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_PRIVILEGE_RESTRICTION_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_ROW_ACCESS_POLICY_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_ALL_ROW_ACCESS_POLICIES_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_CONSTANT_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_FUNCTION_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ARGUMENT_DEF: _ClassVar[ResolvedNodeKind]
    RESOLVED_ARGUMENT_REF: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_TABLE_FUNCTION_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_RELATION_ARGUMENT_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_ARGUMENT_LIST: _ClassVar[ResolvedNodeKind]
    RESOLVED_FUNCTION_SIGNATURE_HOLDER: _ClassVar[ResolvedNodeKind]
    RESOLVED_DROP_FUNCTION_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_DROP_TABLE_FUNCTION_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CALL_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_IMPORT_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_MODULE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_AGGREGATE_HAVING_MODIFIER: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_MATERIALIZED_VIEW_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_APPROX_VIEW_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_PROCEDURE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_EXECUTE_IMMEDIATE_ARGUMENT: _ClassVar[ResolvedNodeKind]
    RESOLVED_EXECUTE_IMMEDIATE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ASSIGNMENT_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_ENTITY_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_ENTITY_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_PIVOT_COLUMN: _ClassVar[ResolvedNodeKind]
    RESOLVED_PIVOT_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_RETURNING_CLAUSE: _ClassVar[ResolvedNodeKind]
    RESOLVED_UNPIVOT_ARG: _ClassVar[ResolvedNodeKind]
    RESOLVED_UNPIVOT_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_MATCH_RECOGNIZE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_MEASURE_GROUP: _ClassVar[ResolvedNodeKind]
    RESOLVED_MATCH_RECOGNIZE_VARIABLE_DEFINITION: _ClassVar[ResolvedNodeKind]
    RESOLVED_MATCH_RECOGNIZE_PATTERN_EMPTY: _ClassVar[ResolvedNodeKind]
    RESOLVED_MATCH_RECOGNIZE_PATTERN_ANCHOR: _ClassVar[ResolvedNodeKind]
    RESOLVED_MATCH_RECOGNIZE_PATTERN_VARIABLE_REF: _ClassVar[ResolvedNodeKind]
    RESOLVED_MATCH_RECOGNIZE_PATTERN_OPERATION: _ClassVar[ResolvedNodeKind]
    RESOLVED_MATCH_RECOGNIZE_PATTERN_QUANTIFICATION: _ClassVar[ResolvedNodeKind]
    RESOLVED_CLONE_DATA_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_TABLE_AND_COLUMN_INFO: _ClassVar[ResolvedNodeKind]
    RESOLVED_ANALYZE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_AUX_LOAD_DATA_PARTITION_FILTER: _ClassVar[ResolvedNodeKind]
    RESOLVED_AUX_LOAD_DATA_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_PROPERTY_GRAPH_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_ELEMENT_TABLE: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_NODE_TABLE_REFERENCE: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_ELEMENT_LABEL: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_PROPERTY_DECLARATION: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_PROPERTY_DEFINITION: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_DYNAMIC_LABEL_SPECIFICATION: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_DYNAMIC_PROPERTIES_SPECIFICATION: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_REF_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_LINEAR_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_TABLE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_CALL_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_PATH_PATTERN_QUANTIFIER: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_PATH_SEARCH_PREFIX: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_NODE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_EDGE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_GET_ELEMENT_PROPERTY: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_LABEL_NARY_EXPR: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_LABEL: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_WILD_CARD_LABEL: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_ELEMENT_IDENTIFIER: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_ELEMENT_PROPERTY: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_MAKE_ELEMENT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ARRAY_AGGREGATE: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_MAKE_ARRAY_VARIABLE: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_PATH_MODE: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_PATH_COST: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_PATH_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_GRAPH_IS_LABELED_PREDICATE: _ClassVar[ResolvedNodeKind]
    RESOLVED_UNDROP_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_IDENTITY_COLUMN_INFO: _ClassVar[ResolvedNodeKind]
    RESOLVED_DESCRIBE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_STATIC_DESCRIBE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_ASSERT_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_LOG_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_PIPE_IF_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_PIPE_IF_CASE: _ClassVar[ResolvedNodeKind]
    RESOLVED_PIPE_FORK_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_PIPE_TEE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_PIPE_EXPORT_DATA_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_PIPE_CREATE_TABLE_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_PIPE_INSERT_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_SUBPIPELINE: _ClassVar[ResolvedNodeKind]
    RESOLVED_SUBPIPELINE_INPUT_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_SUBPIPELINE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_GENERALIZED_QUERY_SUBPIPELINE: _ClassVar[ResolvedNodeKind]
    RESOLVED_BARRIER_SCAN: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_CONNECTION_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_CONNECTION_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_LOCK_MODE: _ClassVar[ResolvedNodeKind]
    RESOLVED_UPDATE_FIELD_ITEM: _ClassVar[ResolvedNodeKind]
    RESOLVED_UPDATE_CONSTRUCTOR: _ClassVar[ResolvedNodeKind]
    RESOLVED_CREATE_SEQUENCE_STMT: _ClassVar[ResolvedNodeKind]
    RESOLVED_ALTER_SEQUENCE_STMT: _ClassVar[ResolvedNodeKind]
    __ResolvedNodeKind__switch_must_have_default__: _ClassVar[ResolvedNodeKind]
RESOLVED_LITERAL: ResolvedNodeKind
RESOLVED_PARAMETER: ResolvedNodeKind
RESOLVED_EXPRESSION_COLUMN: ResolvedNodeKind
RESOLVED_CATALOG_COLUMN_REF: ResolvedNodeKind
RESOLVED_COLUMN_REF: ResolvedNodeKind
RESOLVED_GROUPING_SET_MULTI_COLUMN: ResolvedNodeKind
RESOLVED_CONSTANT: ResolvedNodeKind
RESOLVED_SYSTEM_VARIABLE: ResolvedNodeKind
RESOLVED_INLINE_LAMBDA: ResolvedNodeKind
RESOLVED_SEQUENCE: ResolvedNodeKind
RESOLVED_FILTER_FIELD_ARG: ResolvedNodeKind
RESOLVED_FILTER_FIELD: ResolvedNodeKind
RESOLVED_FUNCTION_CALL: ResolvedNodeKind
RESOLVED_AGGREGATE_FUNCTION_CALL: ResolvedNodeKind
RESOLVED_ANALYTIC_FUNCTION_CALL: ResolvedNodeKind
RESOLVED_EXTENDED_CAST_ELEMENT: ResolvedNodeKind
RESOLVED_EXTENDED_CAST: ResolvedNodeKind
RESOLVED_CAST: ResolvedNodeKind
RESOLVED_MAKE_STRUCT: ResolvedNodeKind
RESOLVED_MAKE_PROTO: ResolvedNodeKind
RESOLVED_MAKE_PROTO_FIELD: ResolvedNodeKind
RESOLVED_GET_STRUCT_FIELD: ResolvedNodeKind
RESOLVED_GET_PROTO_FIELD: ResolvedNodeKind
RESOLVED_GET_JSON_FIELD: ResolvedNodeKind
RESOLVED_GET_ROW_FIELD: ResolvedNodeKind
RESOLVED_FLATTEN: ResolvedNodeKind
RESOLVED_FLATTENED_ARG: ResolvedNodeKind
RESOLVED_REPLACE_FIELD_ITEM: ResolvedNodeKind
RESOLVED_REPLACE_FIELD: ResolvedNodeKind
RESOLVED_GET_PROTO_ONEOF: ResolvedNodeKind
RESOLVED_SUBQUERY_EXPR: ResolvedNodeKind
RESOLVED_WITH_EXPR: ResolvedNodeKind
RESOLVED_EXECUTE_AS_ROLE_SCAN: ResolvedNodeKind
RESOLVED_MODEL: ResolvedNodeKind
RESOLVED_CONNECTION: ResolvedNodeKind
RESOLVED_DESCRIPTOR: ResolvedNodeKind
RESOLVED_SINGLE_ROW_SCAN: ResolvedNodeKind
RESOLVED_UNSET_ARGUMENT_SCAN: ResolvedNodeKind
RESOLVED_TABLE_SCAN: ResolvedNodeKind
RESOLVED_JOIN_SCAN: ResolvedNodeKind
RESOLVED_ARRAY_SCAN: ResolvedNodeKind
RESOLVED_COLUMN_HOLDER: ResolvedNodeKind
RESOLVED_FILTER_SCAN: ResolvedNodeKind
RESOLVED_GROUPING_CALL: ResolvedNodeKind
RESOLVED_GROUPING_SET_LIST: ResolvedNodeKind
RESOLVED_GROUPING_SET_PRODUCT: ResolvedNodeKind
RESOLVED_GROUPING_SET: ResolvedNodeKind
RESOLVED_ROLLUP: ResolvedNodeKind
RESOLVED_CUBE: ResolvedNodeKind
RESOLVED_AGGREGATE_SCAN: ResolvedNodeKind
RESOLVED_ANONYMIZED_AGGREGATE_SCAN: ResolvedNodeKind
RESOLVED_DIFFERENTIAL_PRIVACY_AGGREGATE_SCAN: ResolvedNodeKind
RESOLVED_AGGREGATION_THRESHOLD_AGGREGATE_SCAN: ResolvedNodeKind
RESOLVED_SET_OPERATION_ITEM: ResolvedNodeKind
RESOLVED_SET_OPERATION_SCAN: ResolvedNodeKind
RESOLVED_ORDER_BY_SCAN: ResolvedNodeKind
RESOLVED_LIMIT_OFFSET_SCAN: ResolvedNodeKind
RESOLVED_WITH_REF_SCAN: ResolvedNodeKind
RESOLVED_ANALYTIC_SCAN: ResolvedNodeKind
RESOLVED_SAMPLE_SCAN: ResolvedNodeKind
RESOLVED_COMPUTED_COLUMN: ResolvedNodeKind
RESOLVED_DEFERRED_COMPUTED_COLUMN: ResolvedNodeKind
RESOLVED_ORDER_BY_ITEM: ResolvedNodeKind
RESOLVED_COLUMN_ANNOTATIONS: ResolvedNodeKind
RESOLVED_GENERATED_COLUMN_INFO: ResolvedNodeKind
RESOLVED_COLUMN_DEFAULT_VALUE: ResolvedNodeKind
RESOLVED_COLUMN_DEFINITION: ResolvedNodeKind
RESOLVED_PRIMARY_KEY: ResolvedNodeKind
RESOLVED_FOREIGN_KEY: ResolvedNodeKind
RESOLVED_CHECK_CONSTRAINT: ResolvedNodeKind
RESOLVED_OUTPUT_COLUMN: ResolvedNodeKind
RESOLVED_OUTPUT_SCHEMA: ResolvedNodeKind
RESOLVED_PROJECT_SCAN: ResolvedNodeKind
RESOLVED_TVFSCAN: ResolvedNodeKind
RESOLVED_GROUP_ROWS_SCAN: ResolvedNodeKind
RESOLVED_FUNCTION_ARGUMENT: ResolvedNodeKind
RESOLVED_EXPLAIN_STMT: ResolvedNodeKind
RESOLVED_STATEMENT_WITH_PIPE_OPERATORS_STMT: ResolvedNodeKind
RESOLVED_QUERY_STMT: ResolvedNodeKind
RESOLVED_GENERALIZED_QUERY_STMT: ResolvedNodeKind
RESOLVED_MULTI_STMT: ResolvedNodeKind
RESOLVED_CREATE_WITH_ENTRY_STMT: ResolvedNodeKind
RESOLVED_CREATE_DATABASE_STMT: ResolvedNodeKind
RESOLVED_INDEX_ITEM: ResolvedNodeKind
RESOLVED_UNNEST_ITEM: ResolvedNodeKind
RESOLVED_CREATE_INDEX_STMT: ResolvedNodeKind
RESOLVED_CREATE_SCHEMA_STMT: ResolvedNodeKind
RESOLVED_CREATE_EXTERNAL_SCHEMA_STMT: ResolvedNodeKind
RESOLVED_CREATE_TABLE_STMT: ResolvedNodeKind
RESOLVED_CREATE_TABLE_AS_SELECT_STMT: ResolvedNodeKind
RESOLVED_CREATE_MODEL_ALIASED_QUERY: ResolvedNodeKind
RESOLVED_CREATE_MODEL_STMT: ResolvedNodeKind
RESOLVED_CREATE_VIEW_STMT: ResolvedNodeKind
RESOLVED_WITH_PARTITION_COLUMNS: ResolvedNodeKind
RESOLVED_CREATE_SNAPSHOT_TABLE_STMT: ResolvedNodeKind
RESOLVED_CREATE_EXTERNAL_TABLE_STMT: ResolvedNodeKind
RESOLVED_EXPORT_MODEL_STMT: ResolvedNodeKind
RESOLVED_EXPORT_DATA_STMT: ResolvedNodeKind
RESOLVED_EXPORT_METADATA_STMT: ResolvedNodeKind
RESOLVED_DEFINE_TABLE_STMT: ResolvedNodeKind
RESOLVED_DESCRIBE_STMT: ResolvedNodeKind
RESOLVED_SHOW_STMT: ResolvedNodeKind
RESOLVED_BEGIN_STMT: ResolvedNodeKind
RESOLVED_SET_TRANSACTION_STMT: ResolvedNodeKind
RESOLVED_COMMIT_STMT: ResolvedNodeKind
RESOLVED_ROLLBACK_STMT: ResolvedNodeKind
RESOLVED_START_BATCH_STMT: ResolvedNodeKind
RESOLVED_RUN_BATCH_STMT: ResolvedNodeKind
RESOLVED_ABORT_BATCH_STMT: ResolvedNodeKind
RESOLVED_DROP_STMT: ResolvedNodeKind
RESOLVED_DROP_MATERIALIZED_VIEW_STMT: ResolvedNodeKind
RESOLVED_DROP_SNAPSHOT_TABLE_STMT: ResolvedNodeKind
RESOLVED_RECURSIVE_REF_SCAN: ResolvedNodeKind
RESOLVED_RECURSION_DEPTH_MODIFIER: ResolvedNodeKind
RESOLVED_RECURSIVE_SCAN: ResolvedNodeKind
RESOLVED_WITH_SCAN: ResolvedNodeKind
RESOLVED_WITH_ENTRY: ResolvedNodeKind
RESOLVED_OPTION: ResolvedNodeKind
RESOLVED_WINDOW_PARTITIONING: ResolvedNodeKind
RESOLVED_WINDOW_ORDERING: ResolvedNodeKind
RESOLVED_WINDOW_FRAME: ResolvedNodeKind
RESOLVED_ANALYTIC_FUNCTION_GROUP: ResolvedNodeKind
RESOLVED_WINDOW_FRAME_EXPR: ResolvedNodeKind
RESOLVED_DMLVALUE: ResolvedNodeKind
RESOLVED_DMLDEFAULT: ResolvedNodeKind
RESOLVED_ASSERT_STMT: ResolvedNodeKind
RESOLVED_ASSERT_ROWS_MODIFIED: ResolvedNodeKind
RESOLVED_ON_CONFLICT_CLAUSE: ResolvedNodeKind
RESOLVED_INSERT_ROW: ResolvedNodeKind
RESOLVED_INSERT_STMT: ResolvedNodeKind
RESOLVED_DELETE_STMT: ResolvedNodeKind
RESOLVED_UPDATE_ITEM: ResolvedNodeKind
RESOLVED_UPDATE_ITEM_ELEMENT: ResolvedNodeKind
RESOLVED_UPDATE_STMT: ResolvedNodeKind
RESOLVED_MERGE_WHEN: ResolvedNodeKind
RESOLVED_MERGE_STMT: ResolvedNodeKind
RESOLVED_TRUNCATE_STMT: ResolvedNodeKind
RESOLVED_OBJECT_UNIT: ResolvedNodeKind
RESOLVED_PRIVILEGE: ResolvedNodeKind
RESOLVED_GRANT_STMT: ResolvedNodeKind
RESOLVED_REVOKE_STMT: ResolvedNodeKind
RESOLVED_ALTER_DATABASE_STMT: ResolvedNodeKind
RESOLVED_ALTER_INDEX_STMT: ResolvedNodeKind
RESOLVED_ALTER_MATERIALIZED_VIEW_STMT: ResolvedNodeKind
RESOLVED_ALTER_APPROX_VIEW_STMT: ResolvedNodeKind
RESOLVED_ALTER_SCHEMA_STMT: ResolvedNodeKind
RESOLVED_ALTER_EXTERNAL_SCHEMA_STMT: ResolvedNodeKind
RESOLVED_ALTER_MODEL_STMT: ResolvedNodeKind
RESOLVED_ALTER_TABLE_STMT: ResolvedNodeKind
RESOLVED_ALTER_VIEW_STMT: ResolvedNodeKind
RESOLVED_SET_OPTIONS_ACTION: ResolvedNodeKind
RESOLVED_ALTER_SUB_ENTITY_ACTION: ResolvedNodeKind
RESOLVED_ADD_SUB_ENTITY_ACTION: ResolvedNodeKind
RESOLVED_DROP_SUB_ENTITY_ACTION: ResolvedNodeKind
RESOLVED_ADD_COLUMN_ACTION: ResolvedNodeKind
RESOLVED_ADD_COLUMN_IDENTIFIER_ACTION: ResolvedNodeKind
RESOLVED_REBUILD_ACTION: ResolvedNodeKind
RESOLVED_ADD_CONSTRAINT_ACTION: ResolvedNodeKind
RESOLVED_DROP_CONSTRAINT_ACTION: ResolvedNodeKind
RESOLVED_DROP_PRIMARY_KEY_ACTION: ResolvedNodeKind
RESOLVED_ALTER_COLUMN_OPTIONS_ACTION: ResolvedNodeKind
RESOLVED_ALTER_COLUMN_DROP_NOT_NULL_ACTION: ResolvedNodeKind
RESOLVED_ALTER_COLUMN_DROP_GENERATED_ACTION: ResolvedNodeKind
RESOLVED_ALTER_COLUMN_SET_GENERATED_ACTION: ResolvedNodeKind
RESOLVED_ALTER_COLUMN_SET_DATA_TYPE_ACTION: ResolvedNodeKind
RESOLVED_ALTER_COLUMN_SET_DEFAULT_ACTION: ResolvedNodeKind
RESOLVED_ALTER_COLUMN_DROP_DEFAULT_ACTION: ResolvedNodeKind
RESOLVED_DROP_COLUMN_ACTION: ResolvedNodeKind
RESOLVED_RENAME_COLUMN_ACTION: ResolvedNodeKind
RESOLVED_SET_AS_ACTION: ResolvedNodeKind
RESOLVED_SET_COLLATE_CLAUSE: ResolvedNodeKind
RESOLVED_ALTER_TABLE_SET_OPTIONS_STMT: ResolvedNodeKind
RESOLVED_RENAME_STMT: ResolvedNodeKind
RESOLVED_CREATE_PRIVILEGE_RESTRICTION_STMT: ResolvedNodeKind
RESOLVED_CREATE_ROW_ACCESS_POLICY_STMT: ResolvedNodeKind
RESOLVED_DROP_PRIVILEGE_RESTRICTION_STMT: ResolvedNodeKind
RESOLVED_DROP_ROW_ACCESS_POLICY_STMT: ResolvedNodeKind
RESOLVED_DROP_INDEX_STMT: ResolvedNodeKind
RESOLVED_GRANT_TO_ACTION: ResolvedNodeKind
RESOLVED_RESTRICT_TO_ACTION: ResolvedNodeKind
RESOLVED_ADD_TO_RESTRICTEE_LIST_ACTION: ResolvedNodeKind
RESOLVED_REMOVE_FROM_RESTRICTEE_LIST_ACTION: ResolvedNodeKind
RESOLVED_FILTER_USING_ACTION: ResolvedNodeKind
RESOLVED_REVOKE_FROM_ACTION: ResolvedNodeKind
RESOLVED_RENAME_TO_ACTION: ResolvedNodeKind
RESOLVED_ALTER_PRIVILEGE_RESTRICTION_STMT: ResolvedNodeKind
RESOLVED_ALTER_ROW_ACCESS_POLICY_STMT: ResolvedNodeKind
RESOLVED_ALTER_ALL_ROW_ACCESS_POLICIES_STMT: ResolvedNodeKind
RESOLVED_CREATE_CONSTANT_STMT: ResolvedNodeKind
RESOLVED_CREATE_FUNCTION_STMT: ResolvedNodeKind
RESOLVED_ARGUMENT_DEF: ResolvedNodeKind
RESOLVED_ARGUMENT_REF: ResolvedNodeKind
RESOLVED_CREATE_TABLE_FUNCTION_STMT: ResolvedNodeKind
RESOLVED_RELATION_ARGUMENT_SCAN: ResolvedNodeKind
RESOLVED_ARGUMENT_LIST: ResolvedNodeKind
RESOLVED_FUNCTION_SIGNATURE_HOLDER: ResolvedNodeKind
RESOLVED_DROP_FUNCTION_STMT: ResolvedNodeKind
RESOLVED_DROP_TABLE_FUNCTION_STMT: ResolvedNodeKind
RESOLVED_CALL_STMT: ResolvedNodeKind
RESOLVED_IMPORT_STMT: ResolvedNodeKind
RESOLVED_MODULE_STMT: ResolvedNodeKind
RESOLVED_AGGREGATE_HAVING_MODIFIER: ResolvedNodeKind
RESOLVED_CREATE_MATERIALIZED_VIEW_STMT: ResolvedNodeKind
RESOLVED_CREATE_APPROX_VIEW_STMT: ResolvedNodeKind
RESOLVED_CREATE_PROCEDURE_STMT: ResolvedNodeKind
RESOLVED_EXECUTE_IMMEDIATE_ARGUMENT: ResolvedNodeKind
RESOLVED_EXECUTE_IMMEDIATE_STMT: ResolvedNodeKind
RESOLVED_ASSIGNMENT_STMT: ResolvedNodeKind
RESOLVED_CREATE_ENTITY_STMT: ResolvedNodeKind
RESOLVED_ALTER_ENTITY_STMT: ResolvedNodeKind
RESOLVED_PIVOT_COLUMN: ResolvedNodeKind
RESOLVED_PIVOT_SCAN: ResolvedNodeKind
RESOLVED_RETURNING_CLAUSE: ResolvedNodeKind
RESOLVED_UNPIVOT_ARG: ResolvedNodeKind
RESOLVED_UNPIVOT_SCAN: ResolvedNodeKind
RESOLVED_MATCH_RECOGNIZE_SCAN: ResolvedNodeKind
RESOLVED_MEASURE_GROUP: ResolvedNodeKind
RESOLVED_MATCH_RECOGNIZE_VARIABLE_DEFINITION: ResolvedNodeKind
RESOLVED_MATCH_RECOGNIZE_PATTERN_EMPTY: ResolvedNodeKind
RESOLVED_MATCH_RECOGNIZE_PATTERN_ANCHOR: ResolvedNodeKind
RESOLVED_MATCH_RECOGNIZE_PATTERN_VARIABLE_REF: ResolvedNodeKind
RESOLVED_MATCH_RECOGNIZE_PATTERN_OPERATION: ResolvedNodeKind
RESOLVED_MATCH_RECOGNIZE_PATTERN_QUANTIFICATION: ResolvedNodeKind
RESOLVED_CLONE_DATA_STMT: ResolvedNodeKind
RESOLVED_TABLE_AND_COLUMN_INFO: ResolvedNodeKind
RESOLVED_ANALYZE_STMT: ResolvedNodeKind
RESOLVED_AUX_LOAD_DATA_PARTITION_FILTER: ResolvedNodeKind
RESOLVED_AUX_LOAD_DATA_STMT: ResolvedNodeKind
RESOLVED_CREATE_PROPERTY_GRAPH_STMT: ResolvedNodeKind
RESOLVED_GRAPH_ELEMENT_TABLE: ResolvedNodeKind
RESOLVED_GRAPH_NODE_TABLE_REFERENCE: ResolvedNodeKind
RESOLVED_GRAPH_ELEMENT_LABEL: ResolvedNodeKind
RESOLVED_GRAPH_PROPERTY_DECLARATION: ResolvedNodeKind
RESOLVED_GRAPH_PROPERTY_DEFINITION: ResolvedNodeKind
RESOLVED_GRAPH_DYNAMIC_LABEL_SPECIFICATION: ResolvedNodeKind
RESOLVED_GRAPH_DYNAMIC_PROPERTIES_SPECIFICATION: ResolvedNodeKind
RESOLVED_GRAPH_REF_SCAN: ResolvedNodeKind
RESOLVED_GRAPH_LINEAR_SCAN: ResolvedNodeKind
RESOLVED_GRAPH_TABLE_SCAN: ResolvedNodeKind
RESOLVED_GRAPH_CALL_SCAN: ResolvedNodeKind
RESOLVED_GRAPH_SCAN: ResolvedNodeKind
RESOLVED_GRAPH_PATH_PATTERN_QUANTIFIER: ResolvedNodeKind
RESOLVED_GRAPH_PATH_SEARCH_PREFIX: ResolvedNodeKind
RESOLVED_GRAPH_NODE_SCAN: ResolvedNodeKind
RESOLVED_GRAPH_EDGE_SCAN: ResolvedNodeKind
RESOLVED_GRAPH_GET_ELEMENT_PROPERTY: ResolvedNodeKind
RESOLVED_GRAPH_LABEL_NARY_EXPR: ResolvedNodeKind
RESOLVED_GRAPH_LABEL: ResolvedNodeKind
RESOLVED_GRAPH_WILD_CARD_LABEL: ResolvedNodeKind
RESOLVED_GRAPH_ELEMENT_IDENTIFIER: ResolvedNodeKind
RESOLVED_GRAPH_ELEMENT_PROPERTY: ResolvedNodeKind
RESOLVED_GRAPH_MAKE_ELEMENT: ResolvedNodeKind
RESOLVED_ARRAY_AGGREGATE: ResolvedNodeKind
RESOLVED_GRAPH_MAKE_ARRAY_VARIABLE: ResolvedNodeKind
RESOLVED_GRAPH_PATH_MODE: ResolvedNodeKind
RESOLVED_GRAPH_PATH_COST: ResolvedNodeKind
RESOLVED_GRAPH_PATH_SCAN: ResolvedNodeKind
RESOLVED_GRAPH_IS_LABELED_PREDICATE: ResolvedNodeKind
RESOLVED_UNDROP_STMT: ResolvedNodeKind
RESOLVED_IDENTITY_COLUMN_INFO: ResolvedNodeKind
RESOLVED_DESCRIBE_SCAN: ResolvedNodeKind
RESOLVED_STATIC_DESCRIBE_SCAN: ResolvedNodeKind
RESOLVED_ASSERT_SCAN: ResolvedNodeKind
RESOLVED_LOG_SCAN: ResolvedNodeKind
RESOLVED_PIPE_IF_SCAN: ResolvedNodeKind
RESOLVED_PIPE_IF_CASE: ResolvedNodeKind
RESOLVED_PIPE_FORK_SCAN: ResolvedNodeKind
RESOLVED_PIPE_TEE_SCAN: ResolvedNodeKind
RESOLVED_PIPE_EXPORT_DATA_SCAN: ResolvedNodeKind
RESOLVED_PIPE_CREATE_TABLE_SCAN: ResolvedNodeKind
RESOLVED_PIPE_INSERT_SCAN: ResolvedNodeKind
RESOLVED_SUBPIPELINE: ResolvedNodeKind
RESOLVED_SUBPIPELINE_INPUT_SCAN: ResolvedNodeKind
RESOLVED_SUBPIPELINE_STMT: ResolvedNodeKind
RESOLVED_GENERALIZED_QUERY_SUBPIPELINE: ResolvedNodeKind
RESOLVED_BARRIER_SCAN: ResolvedNodeKind
RESOLVED_CREATE_CONNECTION_STMT: ResolvedNodeKind
RESOLVED_ALTER_CONNECTION_STMT: ResolvedNodeKind
RESOLVED_LOCK_MODE: ResolvedNodeKind
RESOLVED_UPDATE_FIELD_ITEM: ResolvedNodeKind
RESOLVED_UPDATE_CONSTRUCTOR: ResolvedNodeKind
RESOLVED_CREATE_SEQUENCE_STMT: ResolvedNodeKind
RESOLVED_ALTER_SEQUENCE_STMT: ResolvedNodeKind
__ResolvedNodeKind__switch_must_have_default__: ResolvedNodeKind



"""Protocol buffer definitions for ZetaSQL.

This directory contains the source .proto files from the ZetaSQL project.

For compiled Python modules, see zetasql.wasi._pb2.

To regenerate Python files from these .proto files:
    ./scripts/build_protos.sh
"""



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: zetasql/public/functions/datetime.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'zetasql/public/functions/datetime.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'zetasql/public/functions/datetime.proto\x12\x11zetasql.functions*\xaf\x03\n\x11\x44\x61teTimestampPart\x12\x39\n,__DateTimePart__switch_must_have_a_default__\x10\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\x08\n\x04YEAR\x10\x01\x12\t\n\x05MONTH\x10\x02\x12\x07\n\x03\x44\x41Y\x10\x03\x12\r\n\tDAYOFWEEK\x10\x04\x12\r\n\tDAYOFYEAR\x10\x05\x12\x0b\n\x07QUARTER\x10\x06\x12\x08\n\x04HOUR\x10\x07\x12\n\n\x06MINUTE\x10\x08\x12\n\n\x06SECOND\x10\t\x12\x0f\n\x0bMILLISECOND\x10\n\x12\x0f\n\x0bMICROSECOND\x10\x0b\x12\x0e\n\nNANOSECOND\x10\x0c\x12\x0e\n\nPICOSECOND\x10\x19\x12\x08\n\x04\x44\x41TE\x10\r\x12\x08\n\x04WEEK\x10\x0e\x12\x0c\n\x08\x44\x41TETIME\x10\x0f\x12\x08\n\x04TIME\x10\x10\x12\x0b\n\x07ISOYEAR\x10\x11\x12\x0b\n\x07ISOWEEK\x10\x12\x12\x0f\n\x0bWEEK_MONDAY\x10\x13\x12\x10\n\x0cWEEK_TUESDAY\x10\x14\x12\x12\n\x0eWEEK_WEDNESDAY\x10\x15\x12\x11\n\rWEEK_THURSDAY\x10\x16\x12\x0f\n\x0bWEEK_FRIDAY\x10\x17\x12\x11\n\rWEEK_SATURDAY\x10\x18\x42/\n\x1c\x63om.google.zetasql.functionsB\x0fZetaSQLDateTime')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'zetasql.public.functions.datetime_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.zetasql.functionsB\017ZetaSQLDateTime'
  _globals['_DATETIMESTAMPPART']._serialized_start=63
  _globals['_DATETIMESTAMPPART']._serialized_end=494
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: zetasql/public/type.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'zetasql/public/type.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19zetasql/public/type.proto\x12\x07zetasql\x1a google/protobuf/descriptor.proto\"\xc7\x04\n\tTypeProto\x12$\n\ttype_kind\x18\x01 \x01(\x0e\x32\x11.zetasql.TypeKind\x12+\n\narray_type\x18\x02 \x01(\x0b\x32\x17.zetasql.ArrayTypeProto\x12-\n\x0bstruct_type\x18\x03 \x01(\x0b\x32\x18.zetasql.StructTypeProto\x12+\n\nproto_type\x18\x04 \x01(\x0b\x32\x17.zetasql.ProtoTypeProto\x12)\n\tenum_type\x18\x05 \x01(\x0b\x32\x16.zetasql.EnumTypeProto\x12+\n\nrange_type\x18\x08 \x01(\x0b\x32\x17.zetasql.RangeTypeProto\x12:\n\x12graph_element_type\x18\t \x01(\x0b\x32\x1e.zetasql.GraphElementTypeProto\x12\x34\n\x0fgraph_path_type\x18\x0b \x01(\x0b\x32\x1b.zetasql.GraphPathTypeProto\x12\'\n\x08map_type\x18\n \x01(\x0b\x32\x15.zetasql.MapTypeProto\x12/\n\x0cmeasure_type\x18\x0c \x01(\x0b\x32\x19.zetasql.MeasureTypeProto\x12?\n\x13\x66ile_descriptor_set\x18\x06 \x03(\x0b\x32\".google.protobuf.FileDescriptorSet\x12\x1a\n\x12\x65xtended_type_name\x18\x07 \x01(\t*\n\x08\xa0\x8d\x06\x10\x80\x80\x80\x80\x02\":\n\x0e\x41rrayTypeProto\x12(\n\x0c\x65lement_type\x18\x01 \x01(\x0b\x32\x12.zetasql.TypeProto\"N\n\x10StructFieldProto\x12\x12\n\nfield_name\x18\x01 \x01(\t\x12&\n\nfield_type\x18\x02 \x01(\x0b\x32\x12.zetasql.TypeProto\";\n\x0fStructTypeProto\x12(\n\x05\x66ield\x18\x01 \x03(\x0b\x32\x19.zetasql.StructFieldProto\"\\\n\x0cMapTypeProto\x12$\n\x08key_type\x18\x01 \x01(\x0b\x32\x12.zetasql.TypeProto\x12&\n\nvalue_type\x18\x02 \x01(\x0b\x32\x12.zetasql.TypeProto\"~\n\x0eProtoTypeProto\x12\x12\n\nproto_name\x18\x01 \x01(\t\x12\x17\n\x0fproto_file_name\x18\x02 \x01(\t\x12$\n\x19\x66ile_descriptor_set_index\x18\x03 \x01(\x05:\x01\x30\x12\x19\n\x11\x63\x61talog_name_path\x18\x04 \x03(\t\"\x8e\x01\n\rEnumTypeProto\x12\x11\n\tenum_name\x18\x01 \x01(\t\x12\x16\n\x0e\x65num_file_name\x18\x02 \x01(\t\x12$\n\x19\x66ile_descriptor_set_index\x18\x03 \x01(\x05:\x01\x30\x12\x19\n\x11\x63\x61talog_name_path\x18\x04 \x03(\t\x12\x11\n\tis_opaque\x18\x05 \x01(\x08\":\n\x0eRangeTypeProto\x12(\n\x0c\x65lement_type\x18\x01 \x01(\x0b\x32\x12.zetasql.TypeProto\"5\n\x15OpaqueEnumTypeOptions\x12\x1c\n\x14sql_opaque_enum_name\x18\x01 \x01(\t\"4\n\x16OpaqueEnumValueOptions\x12\x1a\n\x12invalid_enum_value\x18\x01 \x01(\x08\"\xd7\x02\n\x15GraphElementTypeProto\x12\x38\n\x04kind\x18\x01 \x01(\x0e\x32*.zetasql.GraphElementTypeProto.ElementKind\x12G\n\rproperty_type\x18\x02 \x03(\x0b\x32\x30.zetasql.GraphElementTypeProto.PropertyTypeProto\x12\x17\n\x0fgraph_reference\x18\x04 \x03(\t\x12\x12\n\nis_dynamic\x18\x05 \x01(\x08\x1aI\n\x11PropertyTypeProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12&\n\nvalue_type\x18\x02 \x01(\x0b\x32\x12.zetasql.TypeProto\"=\n\x0b\x45lementKind\x12\x10\n\x0cKIND_INVALID\x10\x00\x12\r\n\tKIND_NODE\x10\x01\x12\r\n\tKIND_EDGE\x10\x02J\x04\x08\x03\x10\x04\"z\n\x12GraphPathTypeProto\x12\x31\n\tnode_type\x18\x01 \x01(\x0b\x32\x1e.zetasql.GraphElementTypeProto\x12\x31\n\tedge_type\x18\x02 \x01(\x0b\x32\x1e.zetasql.GraphElementTypeProto\";\n\x10MeasureTypeProto\x12\'\n\x0bresult_type\x18\x01 \x01(\x0b\x32\x12.zetasql.TypeProto*\xe9\x04\n\x08TypeKind\x12\x35\n(__TypeKind__switch_must_have_a_default__\x10\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\x10\n\x0cTYPE_UNKNOWN\x10\x00\x12\x0e\n\nTYPE_INT32\x10\x01\x12\x0e\n\nTYPE_INT64\x10\x02\x12\x0f\n\x0bTYPE_UINT32\x10\x03\x12\x0f\n\x0bTYPE_UINT64\x10\x04\x12\r\n\tTYPE_BOOL\x10\x05\x12\x0e\n\nTYPE_FLOAT\x10\x06\x12\x0f\n\x0bTYPE_DOUBLE\x10\x07\x12\x0f\n\x0bTYPE_STRING\x10\x08\x12\x0e\n\nTYPE_BYTES\x10\t\x12\r\n\tTYPE_DATE\x10\n\x12\x12\n\x0eTYPE_TIMESTAMP\x10\x13\x12\r\n\tTYPE_ENUM\x10\x0f\x12\x0e\n\nTYPE_ARRAY\x10\x10\x12\x0f\n\x0bTYPE_STRUCT\x10\x11\x12\x0e\n\nTYPE_PROTO\x10\x12\x12\r\n\tTYPE_TIME\x10\x14\x12\x11\n\rTYPE_DATETIME\x10\x15\x12\x12\n\x0eTYPE_GEOGRAPHY\x10\x16\x12\x10\n\x0cTYPE_NUMERIC\x10\x17\x12\x13\n\x0fTYPE_BIGNUMERIC\x10\x18\x12\x11\n\rTYPE_EXTENDED\x10\x19\x12\r\n\tTYPE_JSON\x10\x1a\x12\x11\n\rTYPE_INTERVAL\x10\x1b\x12\x12\n\x0eTYPE_TOKENLIST\x10\x1c\x12\x0e\n\nTYPE_RANGE\x10\x1d\x12\x16\n\x12TYPE_GRAPH_ELEMENT\x10\x1e\x12\x13\n\x0fTYPE_GRAPH_PATH\x10!\x12\x0c\n\x08TYPE_MAP\x10\x1f\x12\r\n\tTYPE_UUID\x10 \x12\x10\n\x0cTYPE_MEASURE\x10\"\x12\x0c\n\x08TYPE_ROW\x10$:b\n\x18opaque_enum_type_options\x12\x1c.google.protobuf.EnumOptions\x18\x86\xfa\xa5\xdf\x01 \x01(\x0b\x32\x1e.zetasql.OpaqueEnumTypeOptions:i\n\x19opaque_enum_value_options\x12!.google.protobuf.EnumValueOptions\x18\xa6\x8e\xa6\xe2\x01 \x01(\x0b\x32\x1f.zetasql.OpaqueEnumValueOptionsB$\n\x12\x63om.google.zetasqlB\x0bZetaSQLType\xf8\x01\x01')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'zetasql.public.type_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\022com.google.zetasqlB\013ZetaSQLType\370\001\001'
  _globals['_TYPEKIND']._serialized_start=1927
  _globals['_TYPEKIND']._serialized_end=2544
  _globals['_TYPEPROTO']._serialized_start=73
  _globals['_TYPEPROTO']._serialized_end=656
  _globals['_ARRAYTYPEPROTO']._serialized_start=658
  _globals['_ARRAYTYPEPROTO']._serialized_end=716
  _globals['_STRUCTFIELDPROTO']._serialized_start=718
  _globals['_STRUCTFIELDPROTO']._serialized_end=796
  _globals['_STRUCTTYPEPROTO']._serialized_start=798
  _globals['_STRUCTTYPEPROTO']._serialized_end=857
  _globals['_MAPTYPEPROTO']._serialized_start=859
  _globals['_MAPTYPEPROTO']._serialized_end=951
  _globals['_PROTOTYPEPROTO']._serialized_start=953
  _globals['_PROTOTYPEPROTO']._serialized_end=1079
  _globals['_ENUMTYPEPROTO']._serialized_start=1082
  _globals['_ENUMTYPEPROTO']._serialized_end=1224
  _globals['_RANGETYPEPROTO']._serialized_start=1226
  _globals['_RANGETYPEPROTO']._serialized_end=1284
  _globals['_OPAQUEENUMTYPEOPTIONS']._serialized_start=1286
  _globals['_OPAQUEENUMTYPEOPTIONS']._serialized_end=1339
  _globals['_OPAQUEENUMVALUEOPTIONS']._serialized_start=1341
  _globals['_OPAQUEENUMVALUEOPTIONS']._serialized_end=1393
  _globals['_GRAPHELEMENTTYPEPROTO']._serialized_start=1396
  _globals['_GRAPHELEMENTTYPEPROTO']._serialized_end=1739
  _globals['_GRAPHELEMENTTYPEPROTO_PROPERTYTYPEPROTO']._serialized_start=1597
  _globals['_GRAPHELEMENTTYPEPROTO_PROPERTYTYPEPROTO']._serialized_end=1670
  _globals['_GRAPHELEMENTTYPEPROTO_ELEMENTKIND']._serialized_start=1672
  _globals['_GRAPHELEMENTTYPEPROTO_ELEMENTKIND']._serialized_end=1733
  _globals['_GRAPHPATHTYPEPROTO']._serialized_start=1741
  _globals['_GRAPHPATHTYPEPROTO']._serialized_end=1863
  _globals['_MEASURETYPEPROTO']._serialized_start=1865
  _globals['_MEASURETYPEPROTO']._serialized_end=1924
# @@protoc_insertion_point(module_scope)



"""
Type System Extensions - Domain logic for ZetaSQL types

This module extends the auto-generated Type ProtoModel with domain-specific
helper methods, providing a Java-compatible API for type checking and manipulation.
"""

from dataclasses import dataclass
from typing import TYPE_CHECKING, Optional

from zetasql.types.proto_model.generated import (
    Type as _GeneratedType,
)
from zetasql.types.proto_model.generated import (
    TypeKind,
)

if TYPE_CHECKING:
    from zetasql.types.proto_model import ArrayType, MapType, RangeType, StructType


@dataclass
class Type(_GeneratedType):
    """
    Enhanced Type with domain-specific helper methods.

    This class extends the auto-generated Type ProtoModel with Java-compatible
    helper methods for type checking, type narrowing, and formatting.

    All fields are inherited from the generated Type class. This class only
    adds methods and overrides the type hint for type_kind field.

    Examples:
        >>> from zetasql.types import Type, TypeKind
        >>> t = Type(type_kind=TypeKind.TYPE_INT64)
        >>> t.is_int64()
        True
        >>> t.is_numerical()
        True
        >>> t.type_name()
        'INT64'

        >>> # Array type
        >>> arr = Type(type_kind=TypeKind.TYPE_ARRAY)
        >>> arr.is_array()
        True
    """

    # Override type hint for better IDE support (value still compatible with int)
    type_kind: TypeKind = TypeKind.TYPE_UNKNOWN

    # ========== Type Checking Methods (Java API parity) ==========

    def is_int32(self) -> bool:
        """Returns true if type is INT32."""
        return self.type_kind == TypeKind.TYPE_INT32

    def is_int64(self) -> bool:
        """Returns true if type is INT64."""
        return self.type_kind == TypeKind.TYPE_INT64

    def is_uint32(self) -> bool:
        """Returns true if type is UINT32."""
        return self.type_kind == TypeKind.TYPE_UINT32

    def is_uint64(self) -> bool:
        """Returns true if type is UINT64."""
        return self.type_kind == TypeKind.TYPE_UINT64

    def is_bool(self) -> bool:
        """Returns true if type is BOOL."""
        return self.type_kind == TypeKind.TYPE_BOOL

    def is_float(self) -> bool:
        """Returns true if type is FLOAT."""
        return self.type_kind == TypeKind.TYPE_FLOAT

    def is_double(self) -> bool:
        """Returns true if type is DOUBLE."""
        return self.type_kind == TypeKind.TYPE_DOUBLE

    def is_string(self) -> bool:
        """Returns true if type is STRING."""
        return self.type_kind == TypeKind.TYPE_STRING

    def is_bytes(self) -> bool:
        """Returns true if type is BYTES."""
        return self.type_kind == TypeKind.TYPE_BYTES

    def is_date(self) -> bool:
        """Returns true if type is DATE."""
        return self.type_kind == TypeKind.TYPE_DATE

    def is_timestamp(self) -> bool:
        """Returns true if type is TIMESTAMP."""
        return self.type_kind == TypeKind.TYPE_TIMESTAMP

    def is_time(self) -> bool:
        """Returns true if type is TIME."""
        return self.type_kind == TypeKind.TYPE_TIME

    def is_datetime(self) -> bool:
        """Returns true if type is DATETIME."""
        return self.type_kind == TypeKind.TYPE_DATETIME

    def is_interval(self) -> bool:
        """Returns true if type is INTERVAL."""
        return self.type_kind == TypeKind.TYPE_INTERVAL

    def is_geography(self) -> bool:
        """Returns true if type is GEOGRAPHY."""
        return self.type_kind == TypeKind.TYPE_GEOGRAPHY

    def is_numeric(self) -> bool:
        """Returns true if type is NUMERIC."""
        return self.type_kind == TypeKind.TYPE_NUMERIC

    def is_bignumeric(self) -> bool:
        """Returns true if type is BIGNUMERIC."""
        return self.type_kind == TypeKind.TYPE_BIGNUMERIC

    def is_json(self) -> bool:
        """Returns true if type is JSON."""
        return self.type_kind == TypeKind.TYPE_JSON

    def is_enum(self) -> bool:
        """Returns true if type is ENUM."""
        return self.type_kind == TypeKind.TYPE_ENUM

    def is_array(self) -> bool:
        """Returns true if type is ARRAY."""
        return self.type_kind == TypeKind.TYPE_ARRAY

    def is_struct(self) -> bool:
        """Returns true if type is STRUCT."""
        return self.type_kind == TypeKind.TYPE_STRUCT

    def is_proto(self) -> bool:
        """Returns true if type is PROTO."""
        return self.type_kind == TypeKind.TYPE_PROTO

    def is_range(self) -> bool:
        """Returns true if type is RANGE."""
        return self.type_kind == TypeKind.TYPE_RANGE

    def is_map(self) -> bool:
        """Returns true if type is MAP."""
        return self.type_kind == TypeKind.TYPE_MAP

    def is_uuid(self) -> bool:
        """Returns true if type is UUID."""
        return self.type_kind == TypeKind.TYPE_UUID

    def is_graph_element(self) -> bool:
        """Returns true if type is GRAPH_ELEMENT."""
        return self.type_kind == TypeKind.TYPE_GRAPH_ELEMENT

    def is_graph_path(self) -> bool:
        """Returns true if type is GRAPH_PATH."""
        return self.type_kind == TypeKind.TYPE_GRAPH_PATH

    def is_measure(self) -> bool:
        """Returns true if type is MEASURE."""
        return self.type_kind == TypeKind.TYPE_MEASURE

    # ========== Type Category Methods ==========

    def is_integer(self) -> bool:
        """Returns true for any integer type (signed or unsigned)."""
        return TypeKind(self.type_kind).is_integer()

    def is_signed_integer(self) -> bool:
        """Returns true for signed integer types (INT32, INT64)."""
        return TypeKind(self.type_kind).is_signed_integer()

    def is_unsigned_integer(self) -> bool:
        """Returns true for unsigned integer types (UINT32, UINT64)."""
        return TypeKind(self.type_kind).is_unsigned_integer()

    def is_floating_point(self) -> bool:
        """Returns true for floating point types (FLOAT, DOUBLE)."""
        return TypeKind(self.type_kind).is_floating_point()

    def is_numerical(self) -> bool:
        """Returns true for any numeric type (integer, float, numeric, bignumeric)."""
        return TypeKind(self.type_kind).is_numerical()

    def is_temporal(self) -> bool:
        """Returns true for date/time types."""
        return TypeKind(self.type_kind).is_temporal()

    def is_simple(self) -> bool:
        """Returns true if this is a simple (non-composite) type."""
        return TypeKind(self.type_kind).is_simple()

    def is_composite(self) -> bool:
        """Returns true for composite types (array, struct, map, etc)."""
        return TypeKind(self.type_kind).is_composite()

    # ========== Type Narrowing Methods (Java asArray(), asStruct() pattern) ==========

    def as_array(self) -> Optional["ArrayType"]:
        """
        Returns ArrayType if this is an array type, else None.

        This follows the Java API pattern for safe type narrowing.

        Example:
            >>> if t.is_array():
            ...     arr = t.as_array()
            ...     elem_type = arr.element_type
        """
        return self.array_type if self.is_array() else None

    def as_struct(self) -> Optional["StructType"]:
        """
        Returns StructType if this is a struct type, else None.

        Example:
            >>> if t.is_struct():
            ...     st = t.as_struct()
            ...     for field in st.field:
            ...         print(field.field_name)
        """
        return self.struct_type if self.is_struct() else None

    def as_map(self) -> Optional["MapType"]:
        """Returns MapType if this is a map type, else None."""
        return self.map_type if self.is_map() else None

    def as_range(self) -> Optional["RangeType"]:
        """Returns RangeType if this is a range type, else None."""
        return self.range_type if self.is_range() else None

    # ========== Type Formatting ==========

    def type_name(self) -> str:
        """
        Returns human-readable type name.

        Returns:
            Type name string like 'INT64', 'ARRAY<STRING>', 'STRUCT<...>', etc.

        Example:
            >>> Type(type_kind=TypeKind.TYPE_INT64).type_name()
            'INT64'
            >>> # Array type returns 'ARRAY<element_type>'
        """
        # Composite types with nested elements
        if self.is_array() and self.array_type:
            elem_name = self.array_type.element_type.type_name() if self.array_type.element_type else "?"
            return f"ARRAY<{elem_name}>"

        if self.is_struct():
            return "STRUCT<...>"

        if self.is_map() and self.map_type:
            key_name = self.map_type.key_type.type_name() if self.map_type.key_type else "?"
            val_name = self.map_type.value_type.type_name() if self.map_type.value_type else "?"
            return f"MAP<{key_name}, {val_name}>"

        if self.is_range() and self.range_type:
            elem_name = self.range_type.element_type.type_name() if self.range_type.element_type else "?"
            return f"RANGE<{elem_name}>"

        # Simple type - use enum name without TYPE_ prefix
        kind = TypeKind(self.type_kind)
        try:
            return kind.name.removeprefix("TYPE_")
        except (ValueError, AttributeError):
            return f"UNKNOWN({self.type_kind})"

    def __str__(self) -> str:
        """Returns type name for string representation."""
        return self.type_name()

    def __repr__(self) -> str:
        """Returns detailed representation."""
        return f"Type(type_kind={self.type_kind.name if isinstance(self.type_kind, TypeKind) else self.type_kind})"


__all__ = ["Type"]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: zetasql/compliance/compliance_label.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'zetasql/compliance/compliance_label.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from zetasql.wasi._pb2.zetasql.compliance import known_error_pb2 as zetasql_dot_compliance_dot_known__error__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)zetasql/compliance/compliance_label.proto\x12\x07zetasql\x1a$zetasql/compliance/known_error.proto\"N\n\x15\x43omplianceTestsLabels\x12\x35\n\ntest_cases\x18\x01 \x03(\x0b\x32!.zetasql.ComplianceTestCaseLabels\"\xff\x02\n\x18\x43omplianceTestCaseLabels\x12\x11\n\ttest_name\x18\x01 \x01(\t\x12\x12\n\ntest_query\x18\x02 \x01(\t\x12\x36\n\x05param\x18\x07 \x03(\x0b\x32\'.zetasql.ComplianceTestCaseLabels.Param\x12\x30\n\x0ftest_error_mode\x18\x03 \x01(\x0e\x32\x17.zetasql.KnownErrorMode\x12\x19\n\x11\x63ompliance_labels\x18\x04 \x03(\t\x12\x12\n\ntest_shard\x18\x08 \x01(\x03\x12\x41\n\rtest_location\x18\t \x01(\x0b\x32*.zetasql.ComplianceTestCaseLabels.Location\x1a\x38\n\x05Param\x12\x12\n\nparam_name\x18\x01 \x01(\t\x12\x1b\n\x13param_value_literal\x18\x02 \x01(\t\x1a&\n\x08Location\x12\x0c\n\x04\x66ile\x18\x01 \x01(\t\x12\x0c\n\x04line\x18\x02 \x01(\x05')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'zetasql.compliance.compliance_label_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_COMPLIANCETESTSLABELS']._serialized_start=92
  _globals['_COMPLIANCETESTSLABELS']._serialized_end=170
  _globals['_COMPLIANCETESTCASELABELS']._serialized_start=173
  _globals['_COMPLIANCETESTCASELABELS']._serialized_end=556
  _globals['_COMPLIANCETESTCASELABELS_PARAM']._serialized_start=460
  _globals['_COMPLIANCETESTCASELABELS_PARAM']._serialized_end=516
  _globals['_COMPLIANCETESTCASELABELS_LOCATION']._serialized_start=518
  _globals['_COMPLIANCETESTCASELABELS_LOCATION']._serialized_end=556
# @@protoc_insertion_point(module_scope)



"""PreparedExpression wrapper for ZetaSQL expression evaluation.

Provides Java-style PreparedExpression API wrapping the proto-based service.
Mirrors Java PreparedExpression functionality for evaluating SQL expressions.
"""

import contextlib
from typing import TYPE_CHECKING, Any, Optional

from zetasql import types
from zetasql.api.value import Value
from zetasql.core import ZetaSqlLocalService

if TYPE_CHECKING:
    from zetasql.types import TypeKind


class PreparedExpression:
    """Wrapper for prepared SQL expression with Java-style API.

    Mirrors Java PreparedExpression for evaluating SQL expressions with parameters
    and column values. Supports automatic preparation on first execute or explicit
    prepare() call.

    Example:
        >>> from zetasql.api import PreparedExpression, Value
        >>> from zetasql.types import AnalyzerOptions
        >>>
        >>> options = AnalyzerOptions()
        >>> expr = PreparedExpression("1 + 2", options, catalog)
        >>> result = expr.execute()
        >>> assert result.get_int64() == 3
        >>>
        >>> # With parameters
        >>> expr2 = PreparedExpression("@x + @y", options, catalog)
        >>> result = expr2.execute(parameters={"x": Value.int32(10), "y": Value.int32(20)})
        >>> assert result.get_int64() == 30
    """

    def __init__(
        self,
        sql: str,
        options: Optional["types.AnalyzerOptions"] = None,
        catalog: Any | None = None,
        service: ZetaSqlLocalService | None = None,
    ):
        """Initialize PreparedExpression.

        Args:
            sql: SQL expression string
            options: AnalyzerOptions for configuring analysis
            catalog: SimpleCatalog with tables/functions (optional)
        """
        self._sql = sql
        self._options = options
        self._catalog = catalog
        self._service = service or ZetaSqlLocalService.get_instance()
        self._prepared_expression_id: int | None = None
        self._prepared = False
        self._closed = False
        self._output_type: types.Type | None = None
        self._referenced_columns = []
        self._referenced_parameters = []

    @property
    def output_type(self) -> "types.Type":
        """Get the output type of this expression.

        Automatically prepares the expression if not yet prepared.

        Returns:
            Type of the expression result

        Raises:
            RuntimeError: If closed
        """
        if self._closed:
            raise RuntimeError("Expression has been closed")
        if not self._prepared:
            self._prepare()
        return self._output_type

    def get_referenced_columns(self):
        """Get list of column names referenced in this expression.

        Returns:
            List of column names (lowercase)
        """
        if not self._prepared:
            raise RuntimeError("Expression not yet prepared")
        if self._closed:
            raise RuntimeError("Expression has been closed")
        return self._referenced_columns

    def get_referenced_parameters(self):
        """Get list of parameters referenced in this expression.

        Returns:
            List of parameter names (lowercase)
        """
        if not self._prepared:
            raise RuntimeError("Expression not yet prepared")
        if self._closed:
            raise RuntimeError("Expression has been closed")
        return self._referenced_parameters

    def _prepare(self, options: Optional["types.AnalyzerOptions"] = None) -> "PreparedExpression":
        """Explicitly prepare the expression for execution.

        Args:
            options: AnalyzerOptions (overrides constructor options if provided)

        Returns:
            Self for chaining

        Raises:
            RuntimeError: If already closed
            ServerError: On preparation error
        """
        if self._prepared:
            return self  # Already prepared
        if self._closed:
            raise RuntimeError("Expression has been closed")

        if options is not None:
            self._options = options

        analyze_options = self._options or types.AnalyzerOptions()

        response = self._service.prepare(sql=self._sql, options=analyze_options, simple_catalog=self._catalog)

        self._prepared_expression_id = response.prepared.prepared_expression_id
        self._output_type = response.prepared.output_type
        self._referenced_columns = list(response.prepared.referenced_columns)
        self._referenced_parameters = list(response.prepared.referenced_parameters)
        self._prepared = True

        return self

    def _add_inferred_types(self, columns: dict[str, Value], parameters: dict[str, Value]) -> None:
        """Add inferred types from column and parameter values to options.

        Args:
            columns: Dictionary mapping column names to Value objects
            parameters: Dictionary mapping parameter names to Value objects
        """
        if self._options is None:
            self._options = types.AnalyzerOptions()

        for name, value in columns.items():
            col_param = types.AnalyzerOptions.QueryParameter(name=name, type=value.get_type())
            if not any(c.name.lower() == name.lower() for c in self._options.expression_columns):
                self._options.expression_columns.append(col_param)

        for name, value in parameters.items():
            param = types.AnalyzerOptions.QueryParameter(name=name, type=value.get_type())
            if not any(p.name.lower() == name.lower() for p in self._options.query_parameters):
                self._options.query_parameters.append(param)

    def execute(self, columns: dict[str, Value] | None = None, parameters: dict[str, Value] | None = None) -> Value:
        """Execute the expression with column/parameter values.

        Supports flexible calling:
        - execute() - no columns/parameters
        - execute(parameters={...}) - parameters only
        - execute(columns={...}, parameters={...}) - both

        Args:
            columns: Dictionary mapping column names to Value objects
            parameters: Dictionary mapping parameter names to Value objects

        Returns:
            Result Value

        Raises:
            ServerError: On evaluation error
        """
        if self._closed:
            raise RuntimeError("Expression has been closed")

        columns = columns or {}
        parameters = parameters or {}

        if not self._prepared:
            self._add_inferred_types(columns, parameters)
            self._prepare()

        column_params = [
            types.EvaluateRequest.Parameter(name=name.lower(), value=value.to_proto())
            for name, value in columns.items()
        ]

        param_params = [
            types.EvaluateRequest.Parameter(name=name.lower(), value=value.to_proto())
            for name, value in parameters.items()
        ]

        response = self._service.evaluate(
            prepared_expression_id=self._prepared_expression_id,
            columns=column_params,
            params=param_params,
        )

        if response.prepared and response.prepared.prepared_expression_id:
            self._prepared_expression_id = response.prepared.prepared_expression_id
            self._output_type = response.prepared.output_type
            self._referenced_columns = list(response.prepared.referenced_columns)
            self._referenced_parameters = list(response.prepared.referenced_parameters)
            self._prepared = True

        return Value.from_proto(response.value)

    def close(self):
        """Release prepared expression resources."""
        if self._prepared and not self._closed and self._prepared_expression_id is not None:
            try:
                self._service.unprepare(prepared_expression_id=self._prepared_expression_id)
            finally:
                self._closed = True

    def __enter__(self):
        """Context manager entry."""
        self._prepare()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Context manager exit."""
        self.close()
        return False

    def __del__(self):
        """Cleanup on deletion."""
        if not self._closed:
            with contextlib.suppress(Exception):
                self.close()

    @staticmethod
    def builder() -> "PreparedExpression.Builder":
        """Create a new PreparedExpression builder.

        Returns:
            Builder instance for fluent construction

        Example:
            >>> expr = (PreparedExpression.builder()
            ...     .expression("@x + @y")
            ...     .options(options)
            ...     .catalog(catalog)
            ...     .build())
        """
        return PreparedExpression.Builder()

    class Builder:
        """Builder for constructing PreparedExpression instances.

        Provides a fluent interface for configuring and building PreparedExpression
        objects, similar to Java's builder pattern.

        Example:
            >>> builder = PreparedExpression.builder()
            >>> expr = (builder
            ...     .expression("@x + @y")
            ...     .options(options)
            ...     .catalog(catalog)
            ...     .build())
        """

        def __init__(self):
            """Initialize empty builder."""
            self._sql: str | None = None
            self._options: types.AnalyzerOptions | None = None
            self._catalog: Any | None = None
            self._service: ZetaSqlLocalService | None = None
            self._columns: dict[str, types.Type] = {}

        def expression(self, sql: str) -> "PreparedExpression.Builder":
            """Set the SQL expression.

            Args:
                sql: SQL expression string

            Returns:
                Self for chaining
            """
            self._sql = sql
            return self

        def options(self, options: "types.AnalyzerOptions") -> "PreparedExpression.Builder":
            """Set analyzer options.

            Args:
                options: AnalyzerOptions for configuring analysis

            Returns:
                Self for chaining
            """
            self._options = options
            return self

        def catalog(self, catalog: Any) -> "PreparedExpression.Builder":
            """Set the catalog.

            Args:
                catalog: SimpleCatalog with tables/functions

            Returns:
                Self for chaining
            """
            self._catalog = catalog
            return self

        def service(self, service: ZetaSqlLocalService) -> "PreparedExpression.Builder":
            """Set the service instance.

            Args:
                service: ZetaSqlLocalService instance

            Returns:
                Self for chaining
            """
            self._service = service
            return self

        def column(self, name: str, type_kind: "TypeKind") -> "PreparedExpression.Builder":
            """Add an expression column definition.

            Args:
                name: Column name
                type_kind: TypeKind for the column

            Returns:
                Self for chaining

            Example:
                >>> builder.column("age", TypeKind.TYPE_INT64)
            """
            self._columns[name] = types.Type(type_kind=type_kind)
            return self

        def build(self) -> "PreparedExpression":
            """Build the PreparedExpression instance.

            Returns:
                Configured PreparedExpression

            Raises:
                ValueError: If required fields are missing
            """
            if self._sql is None:
                raise ValueError("SQL expression is required")

            # Create the PreparedExpression
            expr = PreparedExpression(
                sql=self._sql,
                options=self._options,
                catalog=self._catalog,
                service=self._service,
            )

            # Add column definitions to options if provided
            if self._columns:
                if expr._options is None:
                    expr._options = types.AnalyzerOptions()

                for name, col_type in self._columns.items():
                    col_param = types.AnalyzerOptions.QueryParameter(name=name, type=col_type)
                    # Check if column already exists
                    col_exists = any(c.name.lower() == name.lower() for c in expr._options.expression_columns)
                    if not col_exists:
                        expr._options.expression_columns.append(col_param)

            return expr
